@@ -33,9 +33,14 @@
       systems which are intended to control, monitor, or use the services
       provided by the program in which the class is embedded.<br />
       The emphasis is on making it robust/reliable/simple, so you can rapidly
-      develop software using it.  It is a single-threaded, single-process
+      develop software using it.<br />
+      By default, it is a single-threaded, single-process
       system using asynchronous I/O, so you can easily run it under
       debug in gdb to fix any bugs in your delegate object.<br />
+      For performance, it can also operate as a massively multi-threaded
+      process, with separate I/O threads handling groups of hundreds
+      or thousands of simultaneous connections, and a pool of processing
+      threads handling parsing of incoming requests.
     </p>
     <p>
       The class is controlled by a few straightforward settings and
@@ -97,12 +102,12 @@
     <p>If the simple threading outlined above is not sufficient for your
       appplication, a more agressive threading scheme is available.<br />
       You may call the -setIOThreads:andPool: method to ask the WebServer
-      instance to use threading internally itsself.  In this case the
+      instance to use threading internally itself.  In this case the
       low-level I/O operations will be shared across the specified number
       of I/O threads instead of occurring in the master thread (makes sense
       if you need to handle a very large number of simultaneous connections).
       In addition, the parsing of the incoming HTTP request and the generation
-      of the raw data of the outgoing response are performed usign threads
+      of the raw data of the outgoing response are performed using threads
       from the thread pool, so that the I/O threads can concentrate on the
       low level communications.
     </p>
@@ -117,7 +122,7 @@
   </section>
 </chapter>
 
-   $Date: 2011-12-13 15:11:21 +0000 (Tue, 13 Dec 2011) $ $Revision: 34271 $
+   $Date: 2014-04-28 07:51:36 +0100 (Mon, 28 Apr 2014) $ $Revision: 37818 $
    */ 
 
 #ifndef	INCLUDED_WEBSERVER_H
@@ -130,6 +135,7 @@
 @class	IOThread;
 @class	WebServer;
 @class	WebServerConfig;
+@class	WebServerRequest;
 @class	WebServerResponse;
 @class	NSArray;
 @class	NSCountedSet;
@@ -154,7 +160,7 @@
 
 /**
  * Process the HTTP request whose headers and data are provided in
- * a GSMimeDocument.<br />
+ * a GSMimeDocument subclass.<br />
  * Extra headers are created as follows -
  * <deflist>
  *   <term>x-http-method</term>
@@ -167,7 +173,7 @@
  *     if there was no query.</desc>
  *   <term>x-http-scheme</term>
  *   <desc>Returns the URL scheme used (http or https) to access the server.
- *     This is https if the request arrived on an ancrypted connection or if
+ *     This is https if the request arrived on an encrypted connection or if
  *     the server is configured as being behind a secure proxy.</desc>
  *   <term>x-http-version</term>
  *   <desc>The version from the HTTP request.</desc>
@@ -186,14 +192,14 @@
  *   <desc>The password from the 'authorization' header if the request
  *     supplied HTTP basic authentication.</desc>
  *   <term>x-count-requests</term>
- *   <desc>The number of requests being precessed at the point when
+ *   <desc>The number of requests being processed at the point when
  *      this request started (includes this request).</desc>
  *   <term>x-count-connections</term>
  *   <desc>The number of connections established to the WebServer at the
  *      point when this request started (including the connection this
  *      request arrived on).</desc>
  *   <term>x-count-connected-hosts</term>
- *   <desc>The number of connects hosts (IP addresses) at the point when
+ *   <desc>The number of connected hosts (IP addresses) at the point when
  *      this request started (including the host which sent this request).
  *   </desc>
  *   <term>x-count-host-connections</term>
@@ -223,8 +229,8 @@
  * This method is always called in the master thread of your WebServer
  * instance (usually the main thread of your application).
  */
-- (BOOL) processRequest: (GSMimeDocument*)request
-	       response: (GSMimeDocument*)response
+- (BOOL) processRequest: (WebServerRequest*)request
+	       response: (WebServerResponse*)response
 		    for: (WebServer*)http;
 /**
  * Log an error or warning ... if the delegate does not implement this
@@ -271,6 +277,29 @@
  */
 @interface	NSObject(WebServerDelegate)
 
+/* If your delegate implements this method, it will be called before the
+ * first call to handle a request (ie before -preProcessRequest:response:for:
+ * or -processRequest:response:for:) to provide the delegate with the
+ * request header information and allow it to decide whether the request
+ * body should be processed incrementally (return value is non-zero) or not
+ * (return value is zero).  The returned value is treated as a guide
+ * to how much request data should be buffered at a time.<br />
+ * This method is called <em>before</em> any HTTP basic authentication
+ * is done, and may (if threading is turned on) be called from a thread
+ * other than the master one.<br />
+ * If your delegate turns on incremental parsing for a request, then any
+ * time that more incoming data is read, the web server class will look
+ * at how much data it has and decide (based on the return value from this
+ * method) to call your  -processRequest:response:for: method (preceded
+ * by -preProcessRequest:response:for: if it is implemented) so that you 
+ * can handle the new request data.<br />
+ * Your code can check to see if the request is complete by using the
+ * -isCompletedRequest: method, and can check for the latest data added
+ * to the request body using the * -incrementalDataForRequest: method.
+ */
+- (uint32_t) incrementalRequest: (WebServerRequest*)request
+                            for: (WebServer*)http;
+
 /**
  * If your delegate implements this method, it will be called by the
  * -completedWithResponse: method before the response data is actually
@@ -282,8 +311,8 @@
  * NB. if threading is turned on this method may be called from a thread
  * other than the master one.
  */
-- (void) postProcessRequest: (GSMimeDocument*)request
-	           response: (GSMimeDocument*)response
+- (void) postProcessRequest: (WebServerRequest*)request
+	           response: (WebServerResponse*)response
 		        for: (WebServer*)http;
 
 /**
@@ -301,9 +330,10 @@
  * is done, and may (if threading is turned on) be called from a thread
  * other than the master one.
  */
-- (BOOL) preProcessRequest: (GSMimeDocument*)request
-	          response: (GSMimeDocument*)response
+- (BOOL) preProcessRequest: (WebServerRequest*)request
+	          response: (WebServerResponse*)response
 		       for: (WebServer*)http;
+
 @end
 
 /**
@@ -367,8 +397,8 @@
   NSMutableArray	*_ioThreads;
   GSThreadPool		*_pool;
   WebServerConfig	*_conf;
-  NSArray		*_quiet;
-  NSArray		*_hosts;
+  id		        *_unused1;
+  id		        *_unused2;
   NSDictionary		*_sslConfig;
   BOOL			_accepting;
   BOOL			_doPostProcess;
@@ -378,6 +408,7 @@
   BOOL			_pad1;
   BOOL			_pad2;
   BOOL			_doAudit;
+  BOOL			_doIncremental;
   NSUInteger		_substitutionLimit;
   NSUInteger		_maxConnections;
   NSUInteger		_maxPerHost;
@@ -394,12 +425,16 @@
   id			_xCountRequests;
   id			_xCountConnections;
   id			_xCountConnectedHosts;
+  NSLock                *_userInfoLock;
+  NSMutableDictionary   *_userInfoMap;
+  NSLock                *_incrementalDataLock;
+  NSMutableDictionary   *_incrementalDataMap;
   void			*_reserved;
 }
 
 /** Returns the base URL used by the remote client to send the request.
  */
-+ (NSURL*) baseURLForRequest: (GSMimeDocument*)request;
++ (NSURL*) baseURLForRequest: (WebServerRequest*)request;
 
 /**
  * Same as the instance method of the same name.
@@ -466,8 +501,8 @@
  * location, otherwise arguments description is taken as a local path to
  * be used with the base URL of the request.
  */
-+ (BOOL) redirectRequest: (GSMimeDocument*)request
-		response: (GSMimeDocument*)response
++ (BOOL) redirectRequest: (WebServerRequest*)request
+		response: (WebServerResponse*)response
 		      to: (id)destination;
 
 /**
@@ -499,20 +534,25 @@
  * };
  * </example>
  */
-- (BOOL) accessRequest: (GSMimeDocument*)request
-	      response: (GSMimeDocument*)response;
+- (BOOL) accessRequest: (WebServerRequest*)request
+	      response: (WebServerResponse*)response;
+
+/** Return the address the receiver listens for connections on, or nil
+ * if it is not listening.
+ */
+- (NSString*) address;
 
 /**
  * Instructs the server that the connection handlind the current request
  * should be closed once the response has been sent back to the client.
  */
-- (void) closeConnectionAfter: (GSMimeDocument*)response;
+- (void) closeConnectionAfter: (WebServerResponse*)response;
 
 /**
  * <p>This may only be called in the case where a call to the delegate's
  * [(WebServerDelegate)-processRequest:response:for:] method
  * to process a request returned NO, indicating that the delegate
- * would handle the request in another thread and complete it later.
+ * would handle the request asynchronously and complete it later.
  * </p>
  * <p>In such a case, the thread handling the request in the delegate
  * <em>must</em> call this method upon completion (passing in the same
@@ -521,8 +561,12 @@
  * and that it should now take over the job of sending the response to
  * the client process.
  * </p>
+ * <p>If the -streamData:withResponse: method has been called with the
+ * supplied response object, calling this method terminated the streamed
+ * response to the client.
+ * </p>
  */
-- (void) completedWithResponse: (GSMimeDocument*)response;
+- (void) completedWithResponse: (WebServerResponse*)response;
 
 /** Returns an array containing an object representing each connection
  * currently active for this server instance.
@@ -539,7 +583,7 @@
  * NB. For forms POST-ed using <code>multipart/form-data</code> you don't
  * need to perform any explicit decoding as this will already have been
  * done for you and the decoded form will be presented as the request
- * GSMimeDocument.  The fields of the form will be the component parts
+ * GSMimeDocument subclass.  The fields of the form will be the component parts
  * of the content of the request and can be accessed using the standard
  * GSMimeDocument methods.<br />
  * This method returns the number of fields actually decoded.
@@ -547,6 +591,10 @@
 - (NSUInteger) decodeURLEncodedForm: (NSData*)data
 			       into: (NSMutableDictionary*)dict;
 
+/** Return this web server's delegate.
+ */
+- (id) delegate;
+
 /**         
  * Encode an application/x-www-form-urlencoded form and store its
  * representation in the supplied data object.<br />
@@ -568,6 +616,11 @@
  */
 - (NSString*) escapeHTML: (NSString*)str;
 
+/** Returns a data object containing any data read for the body of a
+ * partially read request since the last call for the same request.
+ */
+- (NSData*) incrementalDataForRequest: (WebServerRequest*)request;
+
 /** Initialises the receiver to run on the processes main thread (as
  * returned by [NSThread+mainThread].
  */
@@ -583,6 +636,12 @@
  */
 - (id) initForThread: (NSThread*)aThread;
 
+/** Returns YES if the request has been completely read, NO if it still
+ * needs more data to be read from the client and parsed before it is
+ * complete (ie incremental parsing is in progress).
+ */
+- (BOOL) isCompletedRequest: (WebServerRequest*)request;
+
 /**
  * Returns YES if the server is for HTTPS (encrypted connections),
  * NO otherwise.
@@ -601,9 +660,10 @@
  * Parameters from the request data are <em>added</em> to any found in the
  * query string.<br />
  * Values provided as <code>multipart/form-data</code> are also available
- * in a more flexible format as the content of the request.
+ * in a more flexible format (see [GSMimeDocument]) as the content of
+ * the request.
  */
-- (NSMutableDictionary*) parameters: (GSMimeDocument*)request;
+- (NSMutableDictionary*) parameters: (WebServerRequest*)request;
 
 /**
  * Returns the index'th data parameter for the specified name.<br />
@@ -650,6 +710,11 @@
 			 from: (NSDictionary*)params
 		      charset: (NSString*)charset;
 
+/** Return the port the receiver listens for connections on, or nil
+ * if it is not listening.
+ */
+- (NSString*) port;
+
 /**
  * Loads a template file from disk and places it in aResponse as content
  * whose mime type is determined from the file extension using the
@@ -659,7 +724,7 @@
  * method.  It's unlikely that this method can be as efficient as a dedicated
  * server.  However this mechanism is adequate for moderate throughputs.
  */
-- (BOOL) produceResponse: (GSMimeDocument*)aResponse
+- (BOOL) produceResponse: (WebServerResponse*)aResponse
 	  fromStaticPage: (NSString*)aPath
 		   using: (NSDictionary*)map;
 
@@ -679,7 +744,7 @@
  * can in fact be any object which responds to the [NSDictionary-objectForKey:]
  * message by returning a string or nil.
  */
-- (BOOL) produceResponse: (GSMimeDocument*)aResponse
+- (BOOL) produceResponse: (WebServerResponse*)aResponse
 	    fromTemplate: (NSString*)aPath
 		   using: (NSDictionary*)map;
 
@@ -735,6 +800,46 @@
  */
 - (void) setDurationLogging: (BOOL)aFlag;
 
+/**
+ * Sets a flag to determine whether the header lines in responses are
+ * folded if they are over 78 characters (off by default).<br />
+ * Some buggy clients don't support folding, but do accept long header
+ * lines, and this compatibility setting may be used to allow such clients
+ * to handle the server's responses (though this may of course break
+ * things for other clients).<br />
+ * This setting applies to any connection established after the setting
+ * is changed.<br />
+ * Because use of this setting could result in a faulty response (one
+ * with a long header) being sent to a client which correctly handles
+ * folded headers, it's also controllable individually for each response,
+ * so the same process can respond both to clients which expect folded
+ * headers and clients which expect long headers (see
+ * [WebServerResponse-setFoldHeaders:]).
+ */
+- (void) setFoldHeaders: (BOOL)aFlag;
+
+/**
+ * Sets the number of threads used to process basic I/O and the size of
+ * the thread pool used by the receiver for handling parsing of incoming
+ * requests, generation of outgoing responses, and pre/post processing
+ * of requests by the delegate.<br />
+ * This defaults to no use of threads.<br />
+ * NB. Since each thread typically uses two file descriptors to handle any
+ * inter-thread message dispatch, enabling threading will use at least two
+ * extra file descriptors per thread ... this may easily cause you
+ * to go beyound the per-process limit imposed by the operating system and
+ * you may wish to configure a smaller connection limit or tune the O/S to
+ * allow more descriptors.
+ */
+- (void) setIOThreads: (NSUInteger)threads andPool: (NSInteger)poolSize;
+
+/**
+ * Sets a flag to determine whether I/O logging is to be performed.<br />
+ * If this is YES then all incoming requests and their responses will
+ * be logged using the [(WebServerDelegate)-webLog:for:] method.
+ */
+- (void) setLogRawIO: (BOOL)aFlag;
+
 /**
  * Sets the maximum size of an uploaded request body.<br />
  * The default is 4M bytes.<br />
@@ -875,7 +980,7 @@
  * This information may be retrieved later using the -userInfoForRequest:
  * method.
  */
-- (void) setUserInfo: (NSObject*)info forRequest: (GSMimeDocument*)request;
+- (void) setUserInfo: (NSObject*)info forRequest: (WebServerRequest*)request;
 
 /**
  * Sets a flag to determine whether verbose logging is to be performed.<br />
@@ -887,6 +992,26 @@
  */
 - (void) setVerbose: (BOOL)aFlag;
 
+/**
+ * <p>This may only be called in the case where a call to the delegate's
+ * [(WebServerDelegate)-processRequest:response:for:] method
+ * to process a request returned NO, indicating that the delegate
+ * would handle the request asynchronously and complete it later.
+ * </p>
+ * <p>In this case the response content should be empty, and instead of
+ * sending the response in one go the server will send the response header
+ * followed by the supplied data.  Subsequent calls to this method using
+ * the same response object will send more data to the client.<br />
+ * The code which calls this method <em>must</em> terminate the sequence
+ * of calls with a call to the -completedWithResponse: method.
+ * </p>
+ * <p>The method returns YES is the data is scheduled for sending to the
+ * client, NO if the client has already dropped the connection and there
+ * is no point attempting to stream more data.
+ * </p>
+ */
+- (BOOL) streamData: (NSData*)data withResponse: (WebServerResponse*)response;
+
 /**
  * Perform substitutions replacing the markup in aTemplate with the
  * values supplied by map and appending the results to the result.<br />
@@ -914,9 +1039,31 @@
  * Retrieves additional user information (previously set using the
  * -setUserInfo:forRequest: method) from a request.
  */
-- (NSObject*) userInfoForRequest: (GSMimeDocument*)request;
+- (NSObject*) userInfoForRequest: (WebServerRequest*)request;
 
 @end
 
+#ifndef WEBSERVERINTERNAL
+/** Do not attempt to subclass the WebServerRequest class to add instance
+ * variables ... the public interface is intended to keep your compiler
+ * happy, but does not guarantee that the instance variable layout is
+ * actually what it seems.
+ */
+@interface      WebServerRequest : GSMimeDocument
+@end
+
+/** Do not attempt to subclass the WebServerResponse class to add instance
+ * variables ... the public interface is intended to keep your compiler
+ * happy, but does not guarantee that the instance variable layout is
+ * actually what it seems.
+ */
+@interface      WebServerResponse : GSMimeDocument
+/** Behaves as [WebServer-setFoldHeaders:] but applies only to the headers
+ * in the receiver.
+ */
+- (void) setFoldHeaders: (BOOL)aFlag;
+@end
+#endif
+
 #endif
 
